//
// alternative for : UM7206 universal module
//                   http://www.cc-concepts.com/products/x10/
//                   http://www.appdig.com/adicon_new/rly8xa.htm
//
//
//
// used code from http://www.blitzlogic.com/x10.htm
//
// used PIC16F84 10Mhz 
//
// used boostc compiler http://www.sourceboost.com/
//
//
//      Connect B0 to TW523/XM10 pin 1                 
//              B1                   3                
//              GND                  2                
//
//      connect led to A0
//      connect 5V reed relay via BC547B to A1
//

#include <system.h>

//Target PIC16F84 configuration word
#pragma DATA _CONFIG, _PWRTE_OFF & _WDT_OFF & _HS_OSC & _CP_OFF

//Set clock frequency
#pragma CLOCK_FREQ	10000000

#define X10_ZERO_CROSS  portb.0
#define X10_TO_PIC      portb.1

rom char *X10_HOUSE_CODES= {'M','N','O','P','C','D','A','B','E','F','G','H','K','L','I','J'};
                                  
rom char *X10_KEY_CODES = {13,14,15,16,3 ,4 ,1 ,2 ,5 ,6 ,7 , 8, 11,12,9 ,10};
                         //0  1  2  3  4  5  6  7  8  9  10  11 12 13 14 15

void wait_for_zero_cross() 
{
  if(X10_ZERO_CROSS)
  {
    while(X10_ZERO_CROSS);
  }
  else
  {
    while(!X10_ZERO_CROSS);
  }
}

void x10_read_bits(char n,char *data1,char *data2) 
{
   char i;

   *data1=0;
   *data2=0;

   for(i=1;i<=n;++i) 
   {
     *data1 >>= 1;
     
     if (X10_TO_PIC) 
       clear_bit(*data1,7);
     else
       set_bit(*data1,7);
           
     wait_for_zero_cross();
     delay_100us(4);
     
     *data2 >>= 1;
     
     if (!X10_TO_PIC) 
       clear_bit(*data2,7);
     else
       set_bit(*data2,7);

     wait_for_zero_cross();
     delay_100us(4);
       
   }
   *data1 >>= (8-n);
   *data2 >>= (8-n);
   
}

char x10_read(char *house_code,char *key_code) 
{
	char data1,data2;

  x10_read_bits(2,&data1,&data2); // start of message
  if ((data1!=3)&&(data2!=1)) // error
   return 1;
  
  x10_read_bits(4,house_code,&data2);
  if (*house_code!=data2)// error
    return 1;
  *house_code=X10_HOUSE_CODES[*house_code];
  
  x10_read_bits(5,key_code,&data2);
  if (*key_code != data2)// error
    return 1;
  
  if(*key_code<16)
    *key_code=X10_KEY_CODES[*key_code];
    
  return 0;
}

void main( void )
{
	trisa = 0x00; // output
	trisb = 0xFF; // input

	porta = 0x01; // led on a0 off
	portb = 0x00;

  intcon = 0x00;
  option_reg = 0x00; // pull-up resistors on portb : on

  char house_code;
  char key_code;
  
  char number = 0;
  
  delay_ms(500);

	//Endless loop
	while( 1 )
	{	 
		if(!X10_TO_PIC) 
		{
			if (x10_read(&house_code,&key_code)==0)
			{									
				if (house_code == 'M')  // <---- user definable
				{			
					if (key_code > 16)
					{
						if (number == 1) // <---- user definable
						{
							if (key_code == 20)
								porta = 0xfe; // on
							else
								if (key_code == 28)
								 porta = 0x01; // off							
						}
					}
					else
					{
						number = key_code;
					}
				}
			}
			else
			{
				delay_s(2);
			}
		}		
	}
}
